@@ -1,17 +1,19 @@
+from __future__ import absolute_import
+
 import logging
+import os
 import re
 import select
 import subprocess
+import sys
 import threading
 import time
 
 __all__ = [
     'ExternalService',
     'SpawnedService',
-
 ]
 
-
 log = logging.getLogger(__name__)
 
 
@@ -30,7 +32,7 @@ class ExternalService(object):
 
 class SpawnedService(threading.Thread):
     def __init__(self, args=None, env=None):
-        threading.Thread.__init__(self)
+        super(SpawnedService, self).__init__()
 
         if args is None:
             raise TypeError("args parameter is required")
@@ -42,9 +44,7 @@ class SpawnedService(threading.Thread):
         self.should_die = threading.Event()
         self.child = None
         self.alive = False
-
-    def run(self):
-        self.run_with_handles()
+        self.daemon = True
 
     def _spawn(self):
         if self.alive: return
@@ -52,6 +52,7 @@ class SpawnedService(threading.Thread):
 
         self.child = subprocess.Popen(
             self.args,
+            preexec_fn=os.setsid, # to avoid propagating signals
             env=self.env,
             bufsize=1,
             stdout=subprocess.PIPE,
@@ -70,62 +71,53 @@ class SpawnedService(threading.Thread):
         else:
             self.child.kill()
 
-    def run_with_handles(self):
+    def run(self):
         self._spawn()
         while True:
-            (rds, _, _) = select.select([self.child.stdout, self.child.stderr], [], [], 1)
+            try:
+                (rds, _, _) = select.select([self.child.stdout, self.child.stderr], [], [], 1)
+            except select.error as ex:
+                if ex.args[0] == 4:
+                    continue
+                else:
+                    raise
 
             if self.child.stdout in rds:
                 line = self.child.stdout.readline()
-                self.captured_stdout.append(line.decode('utf-8'))
+                self.captured_stdout.append(line.decode('utf-8').rstrip())
 
             if self.child.stderr in rds:
                 line = self.child.stderr.readline()
-                self.captured_stderr.append(line.decode('utf-8'))
+                self.captured_stderr.append(line.decode('utf-8').rstrip())
 
             if self.child.poll() is not None:
                 self.dump_logs()
-                self._spawn()
+                break
 
             if self.should_die.is_set():
                 self._despawn()
                 break
 
     def dump_logs(self):
-        log.critical('stderr')
-        for line in self.captured_stderr:
-            log.critical(line.rstrip())
-
-        log.critical('stdout')
-        for line in self.captured_stdout:
-            log.critical(line.rstrip())
+        sys.stderr.write('\n'.join(self.captured_stderr))
+        sys.stdout.write('\n'.join(self.captured_stdout))
 
     def wait_for(self, pattern, timeout=30):
-        t1 = time.time()
+        start = time.time()
         while True:
-            t2 = time.time()
-            if t2 - t1 >= timeout:
-                try:
-                    self.child.kill()
-                except:
-                    log.exception("Received exception when killing child process")
-                self.dump_logs()
-
+            elapsed = time.time() - start
+            if elapsed >= timeout:
                 log.error("Waiting for %r timed out after %d seconds", pattern, timeout)
                 return False
 
             if re.search(pattern, '\n'.join(self.captured_stdout), re.IGNORECASE) is not None:
-                log.info("Found pattern %r in %d seconds via stdout", pattern, (t2 - t1))
+                log.info("Found pattern %r in %d seconds via stdout", pattern, elapsed)
                 return True
             if re.search(pattern, '\n'.join(self.captured_stderr), re.IGNORECASE) is not None:
-                log.info("Found pattern %r in %d seconds via stderr", pattern, (t2 - t1))
+                log.info("Found pattern %r in %d seconds via stderr", pattern, elapsed)
                 return True
             time.sleep(0.1)
 
-    def start(self):
-        threading.Thread.start(self)
-
     def stop(self):
         self.should_die.set()
         self.join()
-
