@@ -4,41 +4,24 @@ import threading
 import time
 
 import pytest
-import six
+from kafka.vendor import six
 
-from kafka import SimpleClient
 from kafka.conn import ConnectionStates
 from kafka.consumer.group import KafkaConsumer
 from kafka.coordinator.base import MemberState, Generation
 from kafka.structs import TopicPartition
 
-from test.conftest import version
-from test.testutil import random_string
+from test.fixtures import random_string, version
 
 
 def get_connect_str(kafka_broker):
     return kafka_broker.host + ':' + str(kafka_broker.port)
 
 
-@pytest.fixture
-def simple_client(kafka_broker):
-    return SimpleClient(get_connect_str(kafka_broker))
-
-
-@pytest.fixture
-def topic(simple_client):
-    topic = random_string(5)
-    simple_client.ensure_topic_exists(topic)
-    return topic
-
-
 @pytest.mark.skipif(not version(), reason="No KAFKA_VERSION set")
-def test_consumer(kafka_broker, version):
-
+def test_consumer(kafka_broker, topic, version):
+    # The `topic` fixture is included because
     # 0.8.2 brokers need a topic to function well
-    if version >= (0, 8, 2) and version < (0, 9):
-        topic(simple_client(kafka_broker))
-
     consumer = KafkaConsumer(bootstrap_servers=get_connect_str(kafka_broker))
     consumer.poll(500)
     assert len(consumer._client._conns) > 0
