@@ -2,7 +2,8 @@ import os
 import time
 import uuid
 
-from six.moves import range
+import pytest
+from kafka.vendor.six.moves import range
 
 from kafka import (
     SimpleProducer, KeyedProducer,
@@ -10,14 +11,59 @@ from kafka import (
     RoundRobinPartitioner, HashedPartitioner
 )
 from kafka.codec import has_snappy
-from kafka.common import (
-    FetchRequest, ProduceRequest,
-    UnknownTopicOrPartitionError, LeaderNotAvailableError
-)
+from kafka.errors import UnknownTopicOrPartitionError, LeaderNotAvailableError
 from kafka.producer.base import Producer
+from kafka.structs import FetchRequestPayload, ProduceRequestPayload
 
-from test.fixtures import ZookeeperFixture, KafkaFixture
-from test.testutil import KafkaIntegrationTestCase, kafka_versions
+from test.fixtures import ZookeeperFixture, KafkaFixture, version
+from test.testutil import KafkaIntegrationTestCase, kafka_versions, current_offset
+
+
+# TODO: This duplicates a TestKafkaProducerIntegration method temporarily
+# while the migration to pytest is in progress
+def assert_produce_request(client, topic, messages, initial_offset, message_ct,
+                           partition=0):
+    """Verify the correctness of a produce request
+    """
+    produce = ProduceRequestPayload(topic, partition, messages=messages)
+
+    # There should only be one response message from the server.
+    # This will throw an exception if there's more than one.
+    resp = client.send_produce_request([produce])
+    assert_produce_response(resp, initial_offset)
+
+    assert current_offset(client, topic, partition) == initial_offset + message_ct
+
+
+def assert_produce_response(resp, initial_offset):
+    """Verify that a produce response is well-formed
+    """
+    assert len(resp) == 1
+    assert resp[0].error == 0
+    assert resp[0].offset == initial_offset
+
+
+@pytest.mark.skipif(not version(), reason="No KAFKA_VERSION set")
+def test_produce_many_simple(simple_client, topic):
+    """Test multiple produces using the SimpleClient
+    """
+    start_offset = current_offset(simple_client, topic, 0)
+
+    assert_produce_request(
+        simple_client, topic,
+        [create_message(("Test message %d" % i).encode('utf-8'))
+         for i in range(100)],
+        start_offset,
+        100,
+    )
+
+    assert_produce_request(
+        simple_client, topic,
+        [create_message(("Test message %d" % i).encode('utf-8'))
+         for i in range(100)],
+        start_offset+100,
+        100,
+    )
 
 
 class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
@@ -28,7 +74,7 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
             return
 
         cls.zk = ZookeeperFixture.instance()
-        cls.server = KafkaFixture.instance(0, cls.zk.host, cls.zk.port)
+        cls.server = KafkaFixture.instance(0, cls.zk)
 
     @classmethod
     def tearDownClass(cls):  # noqa
@@ -38,25 +84,6 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
         cls.server.close()
         cls.zk.close()
 
-    @kafka_versions("all")
-    def test_produce_many_simple(self):
-        start_offset = self.current_offset(self.topic, 0)
-
-        self.assert_produce_request(
-            [create_message(("Test message %d" % i).encode('utf-8'))
-             for i in range(100)],
-            start_offset,
-            100,
-        )
-
-        self.assert_produce_request(
-            [create_message(("Test message %d" % i).encode('utf-8'))
-             for i in range(100)],
-            start_offset+100,
-            100,
-        )
-
-    @kafka_versions("all")
     def test_produce_10k_simple(self):
         start_offset = self.current_offset(self.topic, 0)
 
@@ -67,7 +94,6 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
             10000,
         )
 
-    @kafka_versions("all")
     def test_produce_many_gzip(self):
         start_offset = self.current_offset(self.topic, 0)
 
@@ -82,7 +108,6 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
             200,
         )
 
-    @kafka_versions("all")
     def test_produce_many_snappy(self):
         self.skipTest("All snappy integration tests fail with nosnappyjava")
         start_offset = self.current_offset(self.topic, 0)
@@ -95,7 +120,6 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
             200,
         )
 
-    @kafka_versions("all")
     def test_produce_mixed(self):
         start_offset = self.current_offset(self.topic, 0)
 
@@ -113,7 +137,6 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
 
         self.assert_produce_request(messages, start_offset, msg_count)
 
-    @kafka_versions("all")
     def test_produce_100k_gzipped(self):
         start_offset = self.current_offset(self.topic, 0)
 
@@ -139,7 +162,12 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
     #   SimpleProducer Tests   #
     ############################
 
-    @kafka_versions("all")
+    def test_simple_producer_new_topic(self):
+        producer = SimpleProducer(self.client)
+        resp = producer.send_messages('new_topic', self.msg('foobar'))
+        self.assert_produce_response(resp, 0)
+        producer.stop()
+
     def test_simple_producer(self):
         partitions = self.client.get_partition_ids_for_topic(self.topic)
         start_offsets = [self.current_offset(self.topic, p) for p in partitions]
@@ -164,17 +192,6 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
 
         producer.stop()
 
-    @kafka_versions("all")
-    def test_produce__new_topic_fails_with_reasonable_error(self):
-        new_topic = 'new_topic_{guid}'.format(guid = str(uuid.uuid4())).encode('utf-8')
-        producer = SimpleProducer(self.client, random_start=False)
-
-        # At first it doesn't exist
-        with self.assertRaises((UnknownTopicOrPartitionError,
-                                LeaderNotAvailableError)):
-            producer.send_messages(new_topic, self.msg("one"))
-
-    @kafka_versions("all")
     def test_producer_random_order(self):
         producer = SimpleProducer(self.client, random_start=True)
         resp1 = producer.send_messages(self.topic, self.msg("one"), self.msg("two"))
@@ -184,7 +201,6 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
         self.assertEqual(resp1[0].partition, resp3[0].partition)
         self.assertNotEqual(resp1[0].partition, resp2[0].partition)
 
-    @kafka_versions("all")
     def test_producer_ordered_start(self):
         producer = SimpleProducer(self.client, random_start=False)
         resp1 = producer.send_messages(self.topic, self.msg("one"), self.msg("two"))
@@ -195,12 +211,11 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
         self.assertEqual(resp2[0].partition, 1)
         self.assertEqual(resp3[0].partition, 0)
 
-    @kafka_versions("all")
     def test_async_simple_producer(self):
         partition = self.client.get_partition_ids_for_topic(self.topic)[0]
         start_offset = self.current_offset(self.topic, partition)
 
-        producer = SimpleProducer(self.client, async=True, random_start=False)
+        producer = SimpleProducer(self.client, async_send=True, random_start=False)
         resp = producer.send_messages(self.topic, self.msg("one"))
         self.assertEqual(len(resp), 0)
 
@@ -210,7 +225,6 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
         self.assert_fetch_offset(partition, start_offset, [ self.msg("one") ])
 
 
-    @kafka_versions("all")
     def test_batched_simple_producer__triggers_by_message(self):
         partitions = self.client.get_partition_ids_for_topic(self.topic)
         start_offsets = [self.current_offset(self.topic, p) for p in partitions]
@@ -220,7 +234,7 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
         batch_interval = 5
         producer = SimpleProducer(
             self.client,
-            async=True,
+            async_send=True,
             batch_send_every_n=batch_messages,
             batch_send_every_t=batch_interval,
             random_start=False)
@@ -278,15 +292,15 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
 
         producer.stop()
 
-    @kafka_versions("all")
     def test_batched_simple_producer__triggers_by_time(self):
+        self.skipTest("Flakey test -- should be refactored or removed")
         partitions = self.client.get_partition_ids_for_topic(self.topic)
         start_offsets = [self.current_offset(self.topic, p) for p in partitions]
 
         batch_interval = 5
         producer = SimpleProducer(
             self.client,
-            async=True,
+            async_send=True,
             batch_send_every_n=100,
             batch_send_every_t=batch_interval,
             random_start=False)
@@ -339,7 +353,7 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
     #   KeyedProducer Tests    #
     ############################
 
-    @kafka_versions("0.8.1", "0.8.1.1", "0.8.2.0")
+    @kafka_versions('>=0.8.1')
     def test_keyedproducer_null_payload(self):
         partitions = self.client.get_partition_ids_for_topic(self.topic)
         start_offsets = [self.current_offset(self.topic, p) for p in partitions]
@@ -361,7 +375,6 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
 
         producer.stop()
 
-    @kafka_versions("all")
     def test_round_robin_partitioner(self):
         partitions = self.client.get_partition_ids_for_topic(self.topic)
         start_offsets = [self.current_offset(self.topic, p) for p in partitions]
@@ -382,7 +395,6 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
 
         producer.stop()
 
-    @kafka_versions("all")
     def test_hashed_partitioner(self):
         partitions = self.client.get_partition_ids_for_topic(self.topic)
         start_offsets = [self.current_offset(self.topic, p) for p in partitions]
@@ -414,19 +426,21 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
 
         producer.stop()
 
-    @kafka_versions("all")
     def test_async_keyed_producer(self):
         partition = self.client.get_partition_ids_for_topic(self.topic)[0]
         start_offset = self.current_offset(self.topic, partition)
 
-        producer = KeyedProducer(self.client, partitioner = RoundRobinPartitioner, async=True)
+        producer = KeyedProducer(self.client,
+                                 partitioner=RoundRobinPartitioner,
+                                 async_send=True,
+                                 batch_send_every_t=1)
 
         resp = producer.send_messages(self.topic, self.key("key1"), self.msg("one"))
         self.assertEqual(len(resp), 0)
 
         # wait for the server to report a new highwatermark
         while self.current_offset(self.topic, partition) == start_offset:
-          time.sleep(0.1)
+            time.sleep(0.1)
 
         self.assert_fetch_offset(partition, start_offset, [ self.msg("one") ])
 
@@ -436,7 +450,6 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
     #   Producer ACK Tests     #
     ############################
 
-    @kafka_versions("all")
     def test_acks_none(self):
         partition = self.client.get_partition_ids_for_topic(self.topic)[0]
         start_offset = self.current_offset(self.topic, partition)
@@ -454,7 +467,6 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
         self.assert_fetch_offset(partition, start_offset, [ self.msg("one") ])
         producer.stop()
 
-    @kafka_versions("all")
     def test_acks_local_write(self):
         partition = self.client.get_partition_ids_for_topic(self.topic)[0]
         start_offset = self.current_offset(self.topic, partition)
@@ -470,7 +482,6 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
 
         producer.stop()
 
-    @kafka_versions("all")
     def test_acks_cluster_commit(self):
         partition = self.client.get_partition_ids_for_topic(self.topic)[0]
         start_offset = self.current_offset(self.topic, partition)
@@ -488,7 +499,7 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
 
     def assert_produce_request(self, messages, initial_offset, message_ct,
                                partition=0):
-        produce = ProduceRequest(self.bytes_topic, partition, messages=messages)
+        produce = ProduceRequestPayload(self.topic, partition, messages=messages)
 
         # There should only be one response message from the server.
         # This will throw an exception if there's more than one.
@@ -506,7 +517,7 @@ class TestKafkaProducerIntegration(KafkaIntegrationTestCase):
         # There should only be one response message from the server.
         # This will throw an exception if there's more than one.
 
-        resp, = self.client.send_fetch_request([ FetchRequest(self.bytes_topic, partition, start_offset, 1024) ])
+        resp, = self.client.send_fetch_request([FetchRequestPayload(self.topic, partition, start_offset, 1024)])
 
         self.assertEqual(resp.error, 0)
         self.assertEqual(resp.partition, partition)
