@@ -3,7 +3,7 @@ import io
 import struct
 
 import pytest
-import six
+from kafka.vendor import six
 
 from kafka.protocol.api import RequestHeader
 from kafka.protocol.commit import GroupCoordinatorRequest
