@@ -3,20 +3,22 @@ from __future__ import absolute_import
 import functools
 import operator
 import os
-import socket
 import time
 import uuid
 
 import pytest
 from . import unittest
 
-from kafka import SimpleClient, create_message
-from kafka.errors import LeaderNotAvailableError, KafkaTimeoutError, InvalidTopicError
-from kafka.structs import OffsetRequestPayload, ProduceRequestPayload, \
-                          NotLeaderForPartitionError, UnknownTopicOrPartitionError, \
-                          FailedPayloadsError
+from kafka import SimpleClient
+from kafka.errors import (
+    LeaderNotAvailableError, KafkaTimeoutError, InvalidTopicError,
+    NotLeaderForPartitionError, UnknownTopicOrPartitionError,
+    FailedPayloadsError
+)
+from kafka.structs import OffsetRequestPayload
 from test.fixtures import random_string, version_str_to_list, version as kafka_version #pylint: disable=wrong-import-order
 
+
 def kafka_versions(*versions):
 
     def construct_lambda(s):
@@ -30,7 +32,7 @@ def kafka_versions(*versions):
             op_str = s[0:2] # >= <=
             v_str = s[2:]
         else:
-            raise ValueError('Unrecognized kafka version / operator: %s' % s)
+            raise ValueError('Unrecognized kafka version / operator: %s' % (s,))
 
         op_map = {
             '=': operator.eq,
@@ -63,32 +65,6 @@ def kafka_versions(*versions):
 
     return real_kafka_versions
 
-def get_open_port():
-    sock = socket.socket()
-    sock.bind(("", 0))
-    port = sock.getsockname()[1]
-    sock.close()
-    return port
-
-_MESSAGES = {}
-def msg(message):
-    """Format, encode and deduplicate a message
-    """
-    global _MESSAGES #pylint: disable=global-statement
-    if message not in _MESSAGES:
-        _MESSAGES[message] = '%s-%s' % (message, str(uuid.uuid4()))
-
-    return _MESSAGES[message].encode('utf-8')
-
-def send_messages(client, topic, partition, messages):
-    """Send messages to a topic's partition
-    """
-    messages = [create_message(msg(str(m))) for m in messages]
-    produce = ProduceRequestPayload(topic, partition, messages=messages)
-    resp, = client.send_produce_request([produce])
-    assert resp.error == 0
-
-    return [x.value for x in messages]
 
 def current_offset(client, topic, partition, kafka_broker=None):
     """Get the current offset of a topic's partition
@@ -104,6 +80,7 @@ def current_offset(client, topic, partition, kafka_broker=None):
     else:
         return offsets.offsets[0]
 
+
 class KafkaIntegrationTestCase(unittest.TestCase):
     create_client = True
     topic = None
